#    This file is part of the LinuxCNC Machine Builder, a graphical
#    configuration editor for LinuxCNC. Copyright 2013 Arvid Brodin.
#
#    Although all the code is new, this program is influenced by
#    PNCconf (Copyright 2009 Chris Morley), in turn based on stepconf,
#    a graphical configuration editor for linuxcnc (Copyright 2007 Jeff
#    Epler).
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import gtk

class cbx_Hardware:
	def __init__(self, builder, cbx_conns):
		self._cbx = builder.get_object("cbox_hw")
		self._store = gtk.ListStore(str, object)
		self._cbx.set_model(self._store)
		self._renderer = gtk.CellRendererText()
		self._cbx.pack_start(self._renderer)
		self._cbx.add_attribute(self._renderer, "text", 0)
		self._cbx.connect("changed", self.on_change)
		self._cbx_conns = cbx_conns

	def get_selected_pair(self):
		return self._store[self._cbx.get_active()][1]

	def update(self, connectables):
		self._store.clear()
		for pair in connectables:
			# pair = [card, [hwic, ...]]
			self._store.append([str(pair[0]), pair])
		self._cbx.set_active(0)

	def on_change(self, obj):
		index = self._cbx.get_active()
		# This is called on _store.clear(); need to check index
		if index > -1:
			row = self._store[index]
			pair = row[1]
			self._cbx_conns.update(pair[1])


class cbx_Connectors:
	def __init__(self, builder):
		self._cbx = builder.get_object("cbox_conn")
		self._store = gtk.ListStore(str, object)
		self._renderer = gtk.CellRendererText()
		self._cbx.set_model(self._store)
		self._cbx.pack_start(self._renderer)
		self._cbx.add_attribute(self._renderer, "text", 0)

	def update(self, conns):
		self._store.clear()
		for conn in conns:
			self._store.append([str(conn), conn])
		self._cbx.set_active(0)
		if len(conns) < 2:
			self._cbx.set_button_sensitivity(gtk.SENSITIVITY_OFF)
		else:
			self._cbx.set_button_sensitivity(gtk.SENSITIVITY_ON)

	def get_selected(self):
		# Return: instance of _HWInterConn
		return self._store[self._cbx.get_active()][1]


class tvw_Hardware:
	def __init__(self, builder, cbx_hw, HWC):
		self._cbx_hw = cbx_hw
		self._HWC = HWC

		self._tvw = builder.get_object("tvw_hw")
		self._store = gtk.TreeStore(str, object)
		self._tvw.set_model(self._store)

		col = gtk.TreeViewColumn("Hardware tree")
		self._tvw.append_column(col)

		self._renderer = gtk.CellRendererText()
		col.pack_start(self._renderer)
		col.add_attribute(self._renderer, "text", 0)

		selection = self._tvw.get_selection()
		selection.connect("changed", self.on_sel_change)
		selection.set_mode(gtk.SELECTION_BROWSE)

	def on_sel_change(self, obj):
		selection = self._tvw.get_selection()
		(model, iter) = selection.get_selected()
		if iter == None:
			selection.select_path(0)
			(model, iter) = selection.get_selected()
		selected = model.get_value(iter, 1)
		self._cbx_hw.update(selected.get_connectable(self._HWC))

	def add_hw(self, hw):
		selection = self._tvw.get_selection()
		(model, iter) = selection.get_selected()
		iter = self._store.append(iter, [str(hw), hw])
		self._tvw.expand_all()
		selection.select_iter(iter)

	def remove_hw(self):
		selection = self._tvw.get_selection()
		(model, iter) = selection.get_selected()
		self._store.remove(iter)


class btn_Add:
	def __init__(self, builder, cbx_hw, cbx_conn, tvw_hw):
		self._btn = builder.get_object("btn_add")
		self._btn.connect("clicked", self.on_click)
		self._cbx_hw = cbx_hw
		self._cbx_conn = cbx_conn
		self._tvw_hw = tvw_hw

	def on_click(self, obj):
		pair = self._cbx_hw.get_selected_pair()
		# pair[0] is class of selected card; create instance
		hwic = self._cbx_conn.get_selected()
		hw = pair[0](hwic.get_owner())
		hwic.connect(hw)
		self._tvw_hw.add_hw(hw)


class btn_Remove:
	def __init__(self, builder, tvw_hw):
		self._btn = builder.get_object("btn_remove")
		self._btn.connect("clicked", self.on_click)
		self._tvw_hw = tvw_hw

	def on_click(self, obj):
		self._tvw_hw.remove_hw()
