#    This file is part of the LinuxCNC Machine Builder, a graphical
#    configuration editor for LinuxCNC. Copyright 2013 Arvid Brodin.
#
#    Although all the code is new, this program is influenced by
#    PNCconf (Copyright 2009 Chris Morley), in turn based on stepconf,
#    a graphical configuration editor for linuxcnc (Copyright 2007 Jeff
#    Epler).
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from net import Net
from halcomp import halcore


class Function(object):
	def __init__(self, halcomp):
		self._halcomp = halcomp

	### Used by child classes only
	def _net(self, nets, netname):
		halname = self._halcomp.get_pinname(netname)
		dir = self._halcomp.get_pindir(netname)

		if not netname in nets:
			nets[netname] = Net(netname)

		if dir == halcore.PIN_OUT:
			nets[netname].set_pin_out(halname)
		elif dir == halcore.PIN_BIDIR:
			nets[netname].add_pin_bidir(halname)
		elif dir == halcore.PIN_IN:
			nets[netname].add_pin_in(halname)
		else:
			raise Exception("Invalid direction spec")

	def generate_nets(self, nets):
		raise Exception("Abstract method called")

	def get_halcomp(self):
		return self._halcomp


class Joint(Function):
	def __init__(self, halcomp):
		Function.__init__(self, halcomp)

	def generate_nets(self, nets):
		self._net(nets, "pos-cmd")
		self._net(nets, "pos-fb")
		self._net(nets, "enable")
		self._net(nets, "index-enable")
		self._net(nets, "min-lim")
		self._net(nets, "max-lim")
		self._net(nets, "home")


class Controller(Function):
	def __init__(self, halcomp):
		Function.__init__(self, halcomp)

	def generate_nets(self, nets):
		self._net(nets, "pos-cmd")
		self._net(nets, "vel-fb")
		self._net(nets, "pos-fb")
		self._net(nets, "output")
		self._net(nets, "enable")
		self._net(nets, "index-enable")


class Feedback(Function):
	def __init__(self, halcomp):
		Function.__init__(self, halcomp)

	def generate_nets(self, nets):
		self._net(nets, "pos-fb")
		self._net(nets, "vel-fb")
		self._net(nets, "index-enable")
		self._net(nets, "pos-rawcount")


class Command(Function):
	def __init__(self, halcomp):
		Function.__init__(self, halcomp)

	def generate_nets(self, nets):
		self._net(nets, "output")


class MinLim(Function):
	def __init__(self, halcomp):
		Function.__init__(self, halcomp)

	def generate_nets(self, nets):
		self._net(nets, "min-lim")


class MaxLim(Function):
	def __init__(self, halcomp):
		Function.__init__(self, halcomp)

	def generate_nets(self, nets):
		self._net(nets, "max-lim")


class Home(Function):
	def __init__(self, halcomp):
		Function.__init__(self, halcomp)

	def generate_nets(self, nets):
		self._net(nets, "home")
