#    This file is part of the LinuxCNC Machine Builder, a graphical
#    configuration editor for LinuxCNC. Copyright 2013 Arvid Brodin.
#
#    Although all the code is new, this program is influenced by
#    PNCconf (Copyright 2009 Chris Morley), in turn based on stepconf,
#    a graphical configuration editor for linuxcnc (Copyright 2007 Jeff
#    Epler).
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

class _HWInterConn(object):
	def __init__(self, owner, name):
		self._connected = None
		self._owner = owner
		self._name = name

	def is_connected(self):
		return self._connected != None

	def connect(self, hardware):
		if self.is_connected():
			raise Error("Connector %s occupied!" % self._name)
		self._connected = hardware

	def get_connected(self):
		return self._connected

	def get_owner(self):
		return self._owner

	def __repr__(self):
		return self._name


# Downstream interface for the computer being configured
class Direct(_HWInterConn):
	def __init__(self, owner, name):
		_HWInterConn.__init__(self, owner, name)


# Downstream interface used on 5I25, 6I25, ...
class MesaP25(_HWInterConn):
	def __init__(self, owner, name):
		_HWInterConn.__init__(self, owner, name)


# Downstream interface used on 5I20, 5I22, ...
class MesaP50(_HWInterConn):
	def __init__(self, owner, name):
		_HWInterConn.__init__(self, owner, name)


# Downstream interface used on 7I76, 7I77 daughtercards
class MesaRS422(_HWInterConn):
	def __init__(self, owner, name):
		_HWInterConn.__init__(self, owner, name)
