#    This file is part of the LinuxCNC Machine Builder, a graphical
#    configuration editor for LinuxCNC. Copyright 2013 Arvid Brodin.
#
#    Although all the code is new, this program is influenced by
#    PNCconf (Copyright 2009 Chris Morley), in turn based on stepconf,
#    a graphical configuration editor for linuxcnc (Copyright 2007 Jeff
#    Epler).
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from halcomp import halcore
from lcncdefs import AXISCHAR
from halfunc import funccore


class Joint(object):
	def __init__(self, jointno):
		self._jointno = jointno
		self._nets = {}
		self._funcs = {}

	def set_command(self, comp):
		if comp is not None:
			self._funcs["command"] = funccore.Command(comp)

	def set_feedback(self, comp):
		if comp is not None:
			self._funcs["feedback"] = funccore.Feedback(comp)

	def set_minlim(self, comp):
		if comp is not None:
			self._funcs["minlim"] = funccore.MinLim(comp)

	def set_maxlim(self, comp):
		if comp is not None:
			self._funcs["maxlim"] = funccore.MaxLim(comp)

	def set_home(self, comp):
		if comp is not None:
			self._funcs["home"] = funccore.Home(comp)

	def generate_hal(self, file):
		if not "joint" in self._funcs:
			self._funcs["joint"] = funccore.Joint(halcore.Joint(self._jointno))
		if "feedback" in self._funcs:
			self._funcs["controller"] = funccore.Controller(halcore.Pid(self._jointno))

		print >>file
		print >>file, "#*********************************************************"
		if self._jointno != 9:
			print >>file, "#  Joint %d (%s axis)" % (self._jointno,
							AXISCHAR[self._jointno].upper())
		else:
			print >>file, "#  Spindle"
		print >>file, "#*********************************************************"
		print >>file

		for key in self._funcs:
			halcomp = self._funcs[key].get_halcomp()
			halcomp.setup_params(self._jointno)
			if halcomp.generate_params(file) > 0:
				print >>file
			self._funcs[key].generate_nets(self._nets)

		for net in self._nets:
			if self._nets[net].generate_hal(file, "j" + str(self._jointno) + "-") > -1:
				print >>file

	def __ini_item(self, file, item, value):
		print >>file, "%s = %s" % (item, value)

	def generate_ini(self, file):
		if (self._jointno < 3):
			__ini_item(file, "TYPE", "LINEAR")
		else:
			__ini_item(file, "TYPE", "ANGULAR")
