#!/usr/bin/python2
#
#    This file is part of the LinuxCNC Machine Builder, a graphical
#    configuration editor for LinuxCNC. Copyright 2013 Arvid Brodin.
#
#    Although all the code is new, this program is influenced by
#    PNCconf (Copyright 2009 Chris Morley), in turn based on stepconf,
#    a graphical configuration editor for linuxcnc (Copyright 2007 Jeff
#    Epler).
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import sys
try:
	import pygtk
	pygtk.require("2.16")
except:
	pass

try:
	import gtk
	import gtk.glade
except:
	print "GTK not available"
	sys.exit(1)

#from hwic import hwic
import gui.page_hwsel
import gui.page_joint_setup
from hardware import generic
from hardware import mesa_fpga
from halcomp import halcore
from joint import Joint
from machine import Machine


### List of hardware IO device classes
HWC = (mesa_fpga.M5I23, mesa_fpga.M5I25,
	mesa_fpga.M7I44, mesa_fpga.M7I49,
	mesa_fpga.M7I77,
	mesa_fpga.M7I70, mesa_fpga.M7I71,
	generic.EPP)

### Root IO device
platform = generic.Platform()

### GTK Init
builder = gtk.Builder()

win_main = gtk.Window()
vbx_main = gtk.VBox()
win_main.add(vbx_main)
nbk_main = gtk.Notebook()
nbk_main.set_show_tabs(False)
nbk_main.set_show_border(False)
#frame_top = gtk.Frame()
#frame_top.set_shadow_type(gtk.SHADOW_ETCHED_IN)
frame_btm = gtk.Frame()
frame_btm.set_shadow_type(gtk.SHADOW_ETCHED_IN)
vbx_main.pack_start(nbk_main)
builder.add_from_file("gui/bottom_panel.glade")
bottom_panel = builder.get_object("bottom_panel")
bottom_panel.set_border_width(12)
frame_btm.add(bottom_panel)
vbx_main.pack_start(frame_btm, False)

### HW layout page
builder.add_from_file("gui/page_hwsel.glade")
page_hwsel = builder.get_object("page_hwsel")
nbk_main.append_page(page_hwsel, None)

cbx_conns = gui.page_hwsel.cbx_Connectors(builder)
cbx_hw = gui.page_hwsel.cbx_Hardware(builder, cbx_conns)
tvw_hw = gui.page_hwsel.tvw_Hardware(builder, cbx_hw, HWC)
tvw_hw.add_hw(platform)
btn_add = gui.page_hwsel.btn_Add(builder, cbx_hw, cbx_conns, tvw_hw)
btn_remove = gui.page_hwsel.btn_Remove(builder, tvw_hw)


### Joints page
builder.add_from_file("gui/page_joint_setup.glade")

wrp_control = gui.page_joint_setup.Joint_cbx_wrapper(
	builder.get_object("cbx_control"), "(none selected)", generic.TermControl)
wrp_feedback = gui.page_joint_setup.Joint_cbx_wrapper(
	builder.get_object("cbx_feedback"), "Open Loop", generic.TermFeedback)
wrp_neglim = gui.page_joint_setup.Joint_cbx_wrapper(
	builder.get_object("cbx_neglim"), "No negative limit switch", generic.TermInput)
wrp_poslim = gui.page_joint_setup.Joint_cbx_wrapper(
	builder.get_object("cbx_poslim"), "No positive limit switch", generic.TermInput)
wrp_zero = gui.page_joint_setup.Joint_cbx_wrapper(
	builder.get_object("cbx_zero"), "No home switch", generic.TermInput)

joint_widgets = [wrp_control, wrp_feedback, wrp_neglim, wrp_poslim, wrp_zero]
gui_joint_setup = gui.page_joint_setup.Joint_Setup(builder, joint_widgets)
nbk_main.append_page(gui_joint_setup.nbk, None)



### Bottom button panel
def btn_next_clicked(button):
	if nbk_main.get_current_page() == 0:
		for widget in joint_widgets:
			widget.update(platform)

	if nbk_main.get_current_page() == 1:
		# Generate HAL file
		machine = Machine("example")
		for i in range(3):
			joint = Joint(i)
			joint.set_command(wrp_control.get_comp(i))
			joint.set_feedback(wrp_feedback.get_comp(i))
			joint.set_minlim(wrp_neglim.get_comp(i))
			joint.set_maxlim(wrp_poslim.get_comp(i))
			joint.set_home(wrp_zero.get_comp(i))

			machine.add_joint(joint)
		machine.generate_hal()

	nbk_main.next_page()

def btn_prev_clicked(button):
	nbk_main.prev_page()

btn_next = builder.get_object("btn_next")
btn_next.connect("clicked", btn_next_clicked)
btn_prev = builder.get_object("btn_prev")
btn_prev.connect("clicked", btn_prev_clicked)





win_main.show_all()
gtk.main()


